// -*-c++-*-

/*
 * $Id: ParticleSource.h,v 1.1.1.1 2001/12/29 18:28:07 uli Exp $
 *
 * Particle Source
 */

#ifndef _PARTICLESOURCE_H_
#define _PARTICLESOURCE_H_

#include <vector>
#include <osg/Vec3>


/// Particle class.
class Particle;


/**
 * Particle-Source class.
 *
 * A particle source emits (creates) a specific number of particles per second.
 * Various particle parameters can be controlled e.g. position, mass.
 */
class ParticleSource {
public:
    /// Constructor.
    ParticleSource();

    /// Destructor.
    virtual ~ParticleSource() { }

    /// Set particles-per-second.
    void setPerSecond(unsigned int perSecond) { _perSecond = perSecond; }

    /// Get particles-per-second.
    unsigned int getPerSecond() const { return _perSecond; }

    /// Set base position and variance.
    void setPos(const osg::Vec3& pos, const osg::Vec3& posVar) {
        _pos = pos;
        _posVar = posVar;
    }

    /// Set base velocity and variance.
    void setVel(const osg::Vec3& vel, const osg::Vec3& velVar) {
        _vel = vel;
        _velVar = velVar;
    }

    /// Set base life-span and variance.
    void setLifeSpan(float lifeSpan, float lifeSpanVar) {
        _lifeSpan = lifeSpan;
        _lifeSpanVar = lifeSpanVar;
    }

    /// Set base mass and variance.
    void setMass(float mass, float massVar) {
        _mass = mass;
        _massVar = massVar;
    }

    /// Emit particles; calls 'emitParticle'.
    void emit(vector<Particle*>& particle, float dt = 1.0f);

protected:
    /// Particles emitted per second.
    unsigned int _perSecond;

    /// Base position.
    osg::Vec3 _pos;

    /// Position variance.
    osg::Vec3 _posVar;

    /// Base velocity.
    osg::Vec3 _vel;

    /// Velocity variance.
    osg::Vec3 _velVar;

    /// Base life-span.
    float _lifeSpan;

    /// Life-span variance.
    float _lifeSpanVar;

    /// Base mass.
    float _mass;

    /// Mass variance.
    float _massVar;

    /// Emit (create) a new particle.
    virtual void emitParticle(Particle* particle);

    /// Calculate base value +/- variance.
    static float variance(float base, float var);
};

#endif
