// -*-c++-*-

/*
 * $Id: ParticleSystem.cpp,v 1.1.1.1 2001/12/29 18:28:07 uli Exp $
 *
 * Particle System
 */

#include <iostream.h>

#include "ParticleSystem.h"

#include <osg/Statistics>
#include <osg/Timer>


/**********************************************************************
 *
 * Particle System
 *
 **********************************************************************/

// Constructor
ParticleSystem::ParticleSystem(vector<Particle*>* particle) : osg::Drawable()
{
    setSupportsDisplayList(false);
    setUseDisplayList(false);

    _particle = particle;

    _source.clear();
    _action.clear();

    _firstAlive = 0;
    _lastAlive = (*particle).size()-1;

    // Initial dummy bounding box
#define SIZE 3.0f
    _bbox.set(osg::Vec3(-SIZE, -SIZE, -SIZE), osg::Vec3(SIZE, SIZE, SIZE));
#undef SIZE

    srand(time(NULL));
}

// Destructor
ParticleSystem::~ParticleSystem()
{
}

// Return statistics
bool ParticleSystem::getStats(osg::Statistics& stat)
{
    stat.reset();
    stat.setType(osg::Statistics::STAT_PRIMSPERVIEW);
    stat.addNumPrims(_lastAlive - _firstAlive);
    return true;
}

// Add particle source
void ParticleSystem::addSource(ParticleSource* source)
{
    _source.push_back(source);
}

// Add particle action
void ParticleSystem::addAction(ParticleAction* action)
{
    _action.push_back(action);
}

// Update particles
void ParticleSystem::update(float dt)
{
    //osg::Timer_t start;

    // Emit particles
    //start = s_Timer.tick();
    for (unsigned int i = 0; i < _source.size(); i++) {
        _source[i]->emit(*_particle, dt);
    }
    //cerr << "Emit " << s_Timer.delta_m(start, s_Timer.tick()) << endl;

    // Apply actions
    //start = s_Timer.tick();
    for (unsigned int i = 0; i < _action.size(); i++) {
        _action[i]->apply(*_particle, _firstAlive, _lastAlive);
    }
    //cerr << "Action " << s_Timer.delta_m(start, s_Timer.tick()) << endl;

    // Update particles
    _firstAlive = (unsigned int) -1;
    _lastAlive = 0;

    // Bounding box
    _bbox.init();

    //start = s_Timer.tick();
    for (unsigned int i = 0; i < (*_particle).size(); i++) {
        Particle* p = (*_particle)[i];
        if (p->_lifeRemain > 0.0f) {
            p->update(dt);

            // Find first/last alive particle
            if (_firstAlive == (unsigned int) -1)
                _firstAlive = i;
            if (i > _lastAlive)
                _lastAlive = i;

            // Expand bounding box
            _bbox.expandBy(p->_pos);
        }
    }
    //cerr << "Update " << s_Timer.delta_m(start, s_Timer.tick()) << endl;

    // Force bounding box update
    dirtyBound();
}

// Draw particles
void ParticleSystem::drawImmediateMode(osg::State&)
{
    //osg::Timer_t start = s_Timer.tick();
    glDisable(GL_LIGHTING);
    (*_particle)[0]->draw(_particle, _firstAlive, _lastAlive);
    glEnable(GL_LIGHTING);
    //cerr << "Draw " << s_Timer.delta_m(start, s_Timer.tick()) << endl;
}


/**********************************************************************
 *
 * Particle Node
 *
 **********************************************************************/

// AppCallback
class PSUpdateCallback : public osg::NodeCallback {
public:
    /// Constructor.
    PSUpdateCallback(ParticleNode* node = NULL) {
        _lastTime = s_Timer.tick();
        _node = node;
    }

    virtual void operator()(osg::Node* node, osg::NodeVisitor* nv) {
        // Update ParticleSystem
        if (_node) {
            osg::Timer_t now = s_Timer.tick();
            double dt = s_Timer.delta_s(_lastTime, now);
            _lastTime = now;

            for (int i = 0; i < _node->getNumDrawables(); i++) {
                ((ParticleSystem*) _node->getDrawable(i))->update(dt);
            }
        }

        // Continue traversal
        nv->traverse(*node);
    }

private:
    /// Timer
    static osg::Timer s_Timer;

    /// Time stamp
    osg::Timer_t _lastTime;

    /// ParticleNode to update.
    ParticleNode* _node;
};

osg::Timer PSUpdateCallback::s_Timer;

// Constructor
ParticleNode::ParticleNode() : osg::Geode()
{
    setAppCallback(new PSUpdateCallback(this));
}
