// -*-c++-*-

/*
 * $Id: main.cpp,v 1.2 2002/02/24 13:57:57 uli Exp $
 *
 * Particle System test
 */

#include <stdlib.h>
#include <iostream.h>

#include <osgUtil/SceneView>
#include <osgUtil/TrackballManipulator>

#include <osgGLUT/Viewer>
#include <GL/glut.h>

#include <osg/Timer>
#include <osg/Group>
#include <osg/Geode>

#include "ParticleSystem.h"


/**
 * Point particle.
 */
class PointParticle : public Particle {
public:
    /// Constructor.
    PointParticle() : Particle() { }

protected:
    /// Draw single particle or list of particles.
    virtual void draw(vector<Particle*>* particle, unsigned int first, unsigned int last)
    {
        if (particle) {
            // Draw list of particles
            glPointSize(3.0f);
            glEnable(GL_POINT_SMOOTH);
            glBegin(GL_POINTS);
            for (unsigned int i = first; i <= last; i++) {
                Particle* p = (*particle)[i];
                if (p->_lifeRemain > 0.0f) {
#if 1
                    // Draw 'p' particle
                    float c = p->getAge();
                    glColor3f(1.0f, c, c*0.5f);
                    glVertex3fv(p->_pos._v);
#else
                    p->draw();
#endif
                }
            }
            glEnd();
        }
        else {
            // Draw 'this' particle
            float c = getAge();
            glColor3f(1.0f, c, c*0.5f);
            glVertex3fv(_pos._v);
        }
    }
};


/**
 * LineParticle.
 * Subclassed Particle that draws a line from it's last position
 * to the current position.
 */
class LineParticle : public Particle {
public:
    /// Constructor.
    LineParticle() : Particle() { }

    /// Store current position and update.
    virtual void update(float dt = 1.0f) {
        _prevPos = _pos;
        _prevAge = getAge();
        Particle::update(dt);
    }

protected:
    /// Previous position.
    osg::Vec3 _prevPos;

    /// Previous age.
    float _prevAge;

    /// Draw single particle or list of particles.
    virtual void draw(vector<Particle*>* particle, unsigned int first, unsigned int last)
    {
        if (particle) {
            // Draw list of particles
            glBegin(GL_LINES);
            for (unsigned int i = first; i <= last; i++) {
                LineParticle* p = (LineParticle*) (*particle)[i];
                if (p->_lifeRemain > 0.0f) {
#if 1
                    // Draw 'p' particle
                    float c0 = p->_prevAge;
                    float c1 = p->getAge();

                    glColor3f(1.0f, c0, c0 * 0.5f);
                    glVertex3fv(p->_prevPos._v);

                    glColor3f(1.0f, c1, c1 * 0.5f);
                    glVertex3fv(p->_pos._v);
#else
                    p->draw();
#endif
                }
            }
            glEnd();
        }
        else {
            // Draw 'this' particle
            float c0 = _prevAge;
            float c1 = getAge();

            glColor3f(1.0f, c0, c0 * 0.5f);
            glVertex3fv(_prevPos._v);

            glColor3f(1.0f, c1, c1 * 0.5f);
            glVertex3fv(_pos._v);
        }
    }
};


/*
 * Test
 */
int main(int argc, char** argv)
{
    int type = 1;
    unsigned int npart = 500;

    if (argc > 1) {
        if (strcmp(argv[1], "point") == 0) {
            type = 1;
        }
        else if (strcmp(argv[1], "line") == 0) {
            type = 2;
        }
        else {
            cerr << "Usage: " << argv[0] << " [point|line] [<num>]" << endl;
        }
    }
    if (argc > 2) {
        npart = atoi(argv[2]);
    }

    /*
     * Particle system
     */

    // Create particles and setup ParticleSystem
    vector<Particle*> part;
    for (unsigned int i = 0; i < npart; i++) {
        Particle* p = NULL;
        if (type == 1)
            p = new PointParticle;
        else if (type == 2)
            p = new LineParticle;
        part.push_back(p);
    }
    ParticleSystem psys(&part);

    // Create and setup ParticleSource
    ParticleSource* psrc = new ParticleSource;
    psrc->setPerSecond(200);
    //psrc->setPos(osg::Vec3(0.0f, 0.0f, 0.0f), osg::Vec3(0.25f, 0.25f, 0.25f));
    psrc->setVel(osg::Vec3(0.0f, 0.0f, 10.0f), osg::Vec3(2.5f, 2.5f, 3.0f));
    psrc->setLifeSpan(3.5f, 0.5f);
    psrc->setMass(1.0f, 0.5f);
    psys.addSource(psrc);

    // Add Gravity
    psys.addAction(&s_GravityAction);

    // Add ParticleSystem to Geode
    ParticleNode* pnode = new ParticleNode;
    pnode->addDrawable(&psys);

    osg::Group* root = new osg::Group;
    root->addChild(pnode);

    /*
     * Viewer
     */
    glutInit(&argc, argv);

    osgGLUT::Viewer viewer;
    viewer.addViewport(root);
    viewer.registerCameraManipulator(new osgUtil::TrackballManipulator);

    viewer.open();
    viewer.run();

    // Cleanup
    delete psrc;
    for (unsigned int i = 0; i < part.size(); i++) {
        delete part[i];
    }

    return 0;
}

